﻿using System;
using System.ComponentModel;
using System.IO;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Windows.Input;
using Xamarin.Forms;

namespace FreeTextExample
{
    public class MainViewModel
    {
        private bool _isSearchbarVisible = false;
        private bool _isToolbarVisible = true;
        private string _searchedText = string.Empty;
        private bool _isCancelVisible = false;

        public event PropertyChangedEventHandler PropertyChanged;

        void NotifyPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private bool _isHighlightButtonSelected;

        public bool IsHighlightButtonSelected
        {
            get
            {
                return _isHighlightButtonSelected;
            }
            set
            {
                _isHighlightButtonSelected = value;
                NotifyPropertyChanged("IsHighlightButtonSelected");
            }
        }

        private bool _isAddTextButtonSelected;
        public bool IsAddTextButtonSelected
        {
            get
            {
                return _isAddTextButtonSelected;
            }
            set
            {
                _isAddTextButtonSelected = value;
                NotifyPropertyChanged("IsAddTextButtonSelected");
            }
        }

        public ICommand SearchAndToolbarToggleCommand { get; set; }
        public ICommand BottomToolbarBackButtonCommand { get; set; }
        public ICommand FreeTextToolbarButtonCommand { get; set; }

        private Stream _pdfStream;
        public Stream PdfStream
        {
            get
            {
                return _pdfStream ?? (_pdfStream = SetPdfStream());
            }
        }

        public int PageNumber { get; set; }
        public string FileName { get; set; }
        public string ParsedName { get; set; }

        private bool _isBusyIndicatorEnabled = false;
        public bool IsBusyIndicatorEnabled
        {
            get
            {
                return _isBusyIndicatorEnabled;
            }
            set
            {
                _isBusyIndicatorEnabled = value;
            }
        }

        private bool _isFreeTextToolbarVisible;
        public bool IsFreeTextToolbarVisible
        {
            get
            {
                return _isFreeTextToolbarVisible;
            }
            set
            {
                _isFreeTextToolbarVisible = value;
                NotifyPropertyChanged("IsFreeTextToolbarVisible");
            }
        }

        private bool _isHighlightBarVisible;
        public bool IsHighlightBarVisible
        {
            get
            {
                return _isHighlightBarVisible;
            }
            set
            {
                _isHighlightBarVisible = value;
                NotifyPropertyChanged("IsHighlightBarVisible");
            }
        }

        public bool IsSearchbarVisible
        {
            get
            {
                return _isSearchbarVisible;
            }
            set
            {
                _isSearchbarVisible = value;
                NotifyPropertyChanged("IsSearchBarVisible");
            }
        }

        public bool IsToolbarVisible
        {
            get
            {
                return _isToolbarVisible;
            }
            set
            {
                if (_isToolbarVisible == value)
                {
                    return;
                }

                _isToolbarVisible = value;
                NotifyPropertyChanged("IsToolbarVisible");
            }
        }

        public string SearchedText
        {
            get { return _searchedText; }
            set
            {
                _searchedText = value;
                if (_searchedText != string.Empty)
                {
                    IsCancelVisible = true;
                }
                else
                {
                    IsCancelVisible = false;
                }
            }
        }

        public bool IsCancelVisible
        {
            get { return _isCancelVisible; }
            set
            {
                if (_isCancelVisible == value)
                    return;
                _isCancelVisible = value;
            }
        }


        public MainViewModel()
        {
            _isHighlightButtonSelected = false;
            _isAddTextButtonSelected = false;
            SearchAndToolbarToggleCommand = new Command<object>(OnSearchAndToolbarToggleCommand, CanExecute);
            BottomToolbarBackButtonCommand = new Command<object>(OnBottomToolbarBackButtonCommand, CanExecute);
            FreeTextToolbarButtonCommand = new Command<object>(OnFreeTextToolbarButtonCommand, CanExecute);
        }

        private bool CanExecute(object parameter)
        {
            return true;
        }

        private void OnSearchAndToolbarToggleCommand(object destinationPageParam)
        {
            IsToolbarVisible = !IsToolbarVisible;
            IsSearchbarVisible = !IsSearchbarVisible;
            IsHighlightBarVisible = false;
            SearchedText = string.Empty;
        }

        private Stream SetPdfStream()
        {
            return typeof(App).GetTypeInfo().Assembly.GetManifestResourceStream("FreeTextExample.Assets.Series4.pdf");
        }

        private void OnBottomToolbarBackButtonCommand(object parameter)
        {
            IsHighlightBarVisible = false;
            IsFreeTextToolbarVisible = false;
        }

        private void OnFreeTextToolbarButtonCommand(object parameter)
        {
            IsAddTextButtonSelected = !IsAddTextButtonSelected;
        }

    }
}
